package org.embulk.input.filename;

import java.util.List;

import com.google.common.base.Optional;

import org.embulk.config.Config;
import org.embulk.config.ConfigDefault;
import org.embulk.config.ConfigDiff;
import org.embulk.config.ConfigSource;
import org.embulk.config.Task;
import org.embulk.config.TaskReport;
import org.embulk.config.TaskSource;
import org.embulk.spi.Exec;
import org.embulk.spi.OutputPlugin;
import org.embulk.spi.PageOutput;
import org.embulk.spi.Schema;
import org.embulk.spi.Page;
import org.embulk.spi.TransactionalPageOutput;
import org.slf4j.Logger;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;


public class JoinfileOutputPlugin
        implements OutputPlugin
{
    public interface PluginTask
            extends Task
    {

        @Config("path_prefix")
        public String getPathPrefix();


        @Config("file_ext")
        public String getFileExt();
        
        @Config("sum_type")
        @ConfigDefault("filename")
        public String getSumType();

    }

    private final Logger log = Exec.getLogger(getClass());
    
    private static FileOutputStream output = null;
    
    private static String sumType;
    
    @Override
    public ConfigDiff transaction(ConfigSource config,
            Schema schema, int taskCount,
            OutputPlugin.Control control)
    {
        PluginTask task = config.loadConfig(PluginTask.class);

        // retryable (idempotent) output:
        // return resume(task.dump(), schema, taskCount, control);

        // non-retryable (non-idempotent) output:
        
        log.info("In the transaction " + config);
        
        String path = task.getPathPrefix() + task.getFileExt();
        
        sumType = task.getSumType();
        
        log.info("The SumType is: " + sumType);
        try {
          output = new FileOutputStream(new File(path));
        } catch (FileNotFoundException ex) {
          throw new RuntimeException (ex);
        }
        
        
        
        control.run(task.dump());
        
        
        closeFile();
        log.info("In the transaction ");
        return Exec.newConfigDiff();
    }

    @Override
    public ConfigDiff resume(TaskSource taskSource,
            Schema schema, int taskCount,
            OutputPlugin.Control control)
    {
        throw new UnsupportedOperationException("joinfile output plugin does not support resuming");
    }

    @Override
    public void cleanup(TaskSource taskSource,
            Schema schema, int taskCount,
            List<TaskReport> successTaskReports)
    {
    }

    @Override
    public TransactionalPageOutput open(TaskSource taskSource, Schema schema, int taskIndex)
    {
        PluginTask task = taskSource.loadTask(PluginTask.class);
        
        log.info("In the open " + taskSource.toString()+ " # " + taskIndex);
        
        return new TransactionalPageOutput(){
          //private final List<String> filenames = new ArrayList<>() ;
          
          public void add(Page page){
            log.info("The ADD: " + page.getStringReferences() + " ## " +page.getValueReferences());
            try {
              //log.info("The content: " + page.getStringReference(0));
              if (sumType.equals("filename")){
                String line = page.getStringReference(1) + "\n";
                output.write(line.getBytes());
              } else{
                String line = page.getStringReference(0) + "\n";
                output.write(line.getBytes());
              }
              
            } catch (IOException ex) {
              throw new RuntimeException(ex);
            }
          }
          
          public void finish(){
            log.info("Finished");
          }
          
          public void close(){
            log.info("closed");
          }
          
          public void abort(){
            
          }
          
          public TaskReport commit(){
            return Exec.newTaskReport();
            
          }
          
        };
        // Write your code here :)
        //throw new UnsupportedOperationException("JoinfileOutputPlugin.run method is not implemented yet");
    }
    
    public static void closeFile()
    {
      if (output!= null){
        try {
          output.close();
        }catch (IOException ex ) {
          throw new RuntimeException(ex);
        }
      }
    }
}
